#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import with_statement
from contextlib import closing
from zipfile import ZipFile, ZIP_DEFLATED
import sys
import os
import time
import datetime
import re

def getProcessDate(expireDays, dateFrom = datetime.date.today):
    fromdate = dateFrom()
    delta = datetime.timedelta(expireDays)
    return (fromdate - delta)

def isExpired(filename, dateExpired, filetemplate = r'(\d{6})(_\d{2})?\.(xml|rep)', datefmt = "%y%m%d"):
    matcher = re.compile(filetemplate)
    result = matcher.match(filename)
    if result:
        filedate = time.strptime(result.group(1), datefmt)

        if(10 > abs(filedate.tm_year - datetime.date.today().year)):
            fdate = datetime.date(filedate.tm_year, filedate.tm_mon, filedate.tm_mday)
        else:
            fdate = datetime.date(datetime.date.today().year, filedate.tm_mon, filedate.tm_mday)

        return (bool(fdate < dateExpired), fdate)
    else:
        return (False, None)

def ensure_dir(dirname):
    if(None == dirname): return
    if(os.path.isdir(dirname)): return
    if not os.path.exists(dirname):
        os.makedirs(dirname)

def processFilesExpired(path, intemplate = r'(\d{6})(_\d{2})?\.(xml|rep)', ziptempl = "%Y_%m_rep.zip", pathto = "archive", expired = 30, fdatefmt = "%y%m%d"):
    ensure_dir(pathto)
    dateexpired = getProcessDate(expired)
    for apath, dirname, filelist in os.walk(path):
        for filename in filelist:
            result, filedate = isExpired(filename, dateexpired, intemplate, fdatefmt)
            if(result):
                arcpathto = os.path.join(pathto, filedate.strftime("%Y"))
                ensure_dir(arcpathto)
                arcname = os.path.join(arcpathto, filedate.strftime(ziptempl))

                with closing(ZipFile(arcname, "a", ZIP_DEFLATED)) as z:
                    absfn = os.path.join(apath, filename)
                    zfn = absfn[len(path)+len(os.sep):] #XXX: relative path
                    z.write(absfn, zfn)
                    os.remove(absfn)
                    print "file {0:s} moved to {1:s}".format(filename, arcname)

def deleteFilesExpired(path, intemplate = r'.*', expired = 30):
    tsNow = datetime.datetime.now()
    tsDelta = datetime.timedelta(days=expired)
    for apath, dirname, filelist in os.walk(path):
        for filename in filelist:
            matcher = re.compile(intemplate)
            result = matcher.match(filename)
            if result:
                absoluteFilename = os.path.join(apath, filename)
                tsFile = datetime.datetime.fromtimestamp(os.path.getmtime(absoluteFilename))
                if tsNow - tsFile > tsDelta:
                    os.remove(absoluteFilename)
                    print "file {0:s} deleted".format(absoluteFilename)

def main():
    if 4 > len(sys.argv):
        print "usage: {0:s} pathfrom [, templ_rgexp [, ziptempl [, pathto [, expired_days [, filedateformat]]]]]".format(sys.argv[0])
        raise SystemExit(1)

    if 6 < len(sys.argv):
        fdatefmt = sys.argv[6]
    else:
        fdatefmt = "%y%m%d"

    if 5 < len(sys.argv):
        expire = sys.argv[5]
    else:
        expire = 30

    if 4 < len(sys.argv):
        pathto = sys.argv[4]
    else:
        pathto = "archive"


    processFilesExpired(sys.argv[1], sys.argv[2], sys.argv[3], pathto, int(expire), fdatefmt)

if __name__ == "__main__":
    sys.exit(main())

